@@ -71,6 +71,15 @@ int ccid_open_hack_pre(unsigned int reader_index)
 		case OZ776_7772:
 			ccid_descriptor->dwMaxDataRate = 9600;
 			break;
+
+		case ElatecTWN4:
+			/* use a timeout of 400 ms instead of 100 ms in CmdGetSlotStatus()
+			 * used by CreateChannelByNameOrChannel()
+			 * The reader answers after 280 ms if no tag is present */
+		case SCM_SCL011:
+			/* The SCM SCL011 reader needs 350 ms to answer */
+			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT * 4;
+			break;
 	}
 
 	/* CCID */
@@ -437,7 +446,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 					length_res = sizeof(res);
 					if (IFD_SUCCESS == CmdEscape(reader_index, cmd2, sizeof(cmd2), res, &length_res, DEFAULT_COM_READ_TIMEOUT))
 					{
-						DEBUG_COMM("Disable SPE retry counter successfull");
+						DEBUG_COMM("Disable SPE retry counter successful");
 					}
 					else
 					{
@@ -458,6 +467,13 @@ int ccid_open_hack_post(unsigned int reader_index)
 			 */
 			ccid_descriptor->bPINSupport = 0;
 			break;
+		case HID_AVIATOR:
+			/* The chip advertises pinpad but actually doesn't have one */
+			ccid_descriptor->bPINSupport = 0;
+			/* Firmware uses chaining */
+			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
+			break;
 
 #if 0
 		/* SCM SCR331-DI contactless */
@@ -511,6 +527,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 				ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
 			}
 			break;
+
+		case ElatecTWN4:
+		case SCM_SCL011:
+			/* restore default timeout (modified in ccid_open_hack_pre()) */
+			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT;
+			break;
 	}
 
 	/* Gemalto readers may report additional information */
